#
#/* Copyright (c) Mark J. Kilgard, 1994. */
#
#/*
# * (c) Copyright 1993, Silicon Graphics, Inc.
# * ALL RIGHTS RESERVED 
# * Permission to use, copy, modify, and distribute this software for 
# * any purpose and without fee is hereby granted, provided that the above
# * copyright notice appear in all copies and that both the copyright notice
# * and this permission notice appear in supporting documentation, and that 
# * the name of Silicon Graphics, Inc. not be used in advertising
# * or publicity pertaining to distribution of the software without specific,
# * written prior permission. 
# *
# * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
# * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
# * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
# * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
# * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
# * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
# * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
# * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
# * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
# * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
# * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
# * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
# * 
# * US Government Users Restricted Rights 
# * Use, duplication, or disclosure by the Government is subject to
# * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
# * (c)(1)(ii) of the Rights in Technical Data and Computer Software
# * clause at DFARS 252.227-7013 and/or in similar or successor
# * clauses in the FAR or the DOD or NASA FAR Supplement.
# * Unpublished-- rights reserved under the copyright laws of the
# * United States.  Contractor/manufacturer is Silicon Graphics,
# * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
# *
# * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
# */
#/*
# *  nurbs.c
# *  This program shows a NURBS (Non-uniform rational B-splines)
# *  surface, shaped like a heart.
# */

require "opengl"
require "glut"

S_NUMPOINTS=13
S_ORDER=3   
S_NUMKNOTS=(S_NUMPOINTS + S_ORDER)
T_NUMPOINTS=3
T_ORDER=3 
T_NUMKNOTS=(T_NUMPOINTS + T_ORDER)
SQRT2=1.41421356237309504880

#/* initialized local data */

$sknots = [-1.0, -1.0, -1.0, 0.0, 1.0, 2.0, 3.0, 4.0,
      4.0,  5.0,  6.0, 7.0, 8.0, 9.0, 9.0, 9.0];
$tknots = [1.0, 1.0, 1.0, 2.0, 2.0, 2.0];

$ctlpoints = [
[   [4.0,2.0,2.0,1.0],[4.0,1.6,2.5,1.0],[4.0,2.0,3.0,1.0]    ],
[   [5.0,4.0,2.0,1.0],[5.0,4.0,2.5,1.0],[5.0,4.0,3.0,1.0] ],
[   [6.0,5.0,2.0,1.0],[6.0,5.0,2.5,1.0],[6.0,5.0,3.0,1.0] ],
[   [SQRT2*6.0,SQRT2*6.0,SQRT2*2.0,SQRT2],
    [SQRT2*6.0,SQRT2*6.0,SQRT2*2.5,SQRT2],
    [SQRT2*6.0,SQRT2*6.0,SQRT2*3.0,SQRT2]  ],
[   [5.2,6.7,2.0,1.0],[5.2,6.7,2.5,1.0],[5.2,6.7,3.0,1.0]   ],
[   [SQRT2*4.0,SQRT2*6.0,SQRT2*2.0,SQRT2],
    [SQRT2*4.0,SQRT2*6.0,SQRT2*2.5,SQRT2],
    [SQRT2*4.0,SQRT2*6.0,SQRT2*3.0,SQRT2]  ],
[   [4.0,5.2,2.0,1.0],[4.0,4.6,2.5,1.0],[4.0,5.2,3.0,1.0]  ],
[   [SQRT2*4.0,SQRT2*6.0,SQRT2*2.0,SQRT2],
    [SQRT2*4.0,SQRT2*6.0,SQRT2*2.5,SQRT2],
    [SQRT2*4.0,SQRT2*6.0,SQRT2*3.0,SQRT2]  ],
[   [2.8,6.7,2.0,1.0],[2.8,6.7,2.5,1.0],[2.8,6.7,3.0,1.0]   ],
[   [SQRT2*2.0,SQRT2*6.0,SQRT2*2.0,SQRT2],
    [SQRT2*2.0,SQRT2*6.0,SQRT2*2.5,SQRT2],
    [SQRT2*2.0,SQRT2*6.0,SQRT2*3.0,SQRT2]  ],
[   [2.0,5.0,2.0,1.0],[2.0,5.0,2.5,1.0],[2.0,5.0,3.0,1.0] ],
[   [3.0,4.0,2.0,1.0],[3.0,4.0,2.5,1.0],[3.0,4.0,3.0,1.0] ],
[   [4.0,2.0,2.0,1.0],[4.0,1.6,2.5,1.0],[4.0,2.0,3.0,1.0]    ]
];

$theNurb = nil;

#/*  Initialize material property, light source, lighting model, 
# *  and depth buffer.
# */
def myinit
    mat_ambient = [ 1.0, 1.0, 1.0, 1.0 ];
    mat_diffuse = [ 1.0, 0.2, 1.0, 1.0 ];
    mat_specular = [ 1.0, 1.0, 1.0, 1.0 ];
    mat_shininess = [ 50.0 ];

    light0_position = [ 1.0, 0.1, 1.0, 0.0 ];
    light1_position = [ -1.0, 0.1, 1.0, 0.0 ];

    lmodel_ambient = [ 0.3, 0.3, 0.3, 1.0 ];

    GL::Material(GL::FRONT, GL::AMBIENT, mat_ambient);
    GL::Material(GL::FRONT, GL::DIFFUSE, mat_diffuse);
    GL::Material(GL::FRONT, GL::SPECULAR, mat_specular);
    GL::Material(GL::FRONT, GL::SHININESS, mat_shininess);
    GL::Light(GL::LIGHT0, GL::POSITION, light0_position);
    GL::Light(GL::LIGHT1, GL::POSITION, light1_position);
    GL::LightModel(GL::LIGHT_MODEL_AMBIENT, lmodel_ambient);

    GL::Enable(GL::LIGHTING);
    GL::Enable(GL::LIGHT0);
    GL::Enable(GL::LIGHT1);
    GL::DepthFunc(GL::LESS);
    GL::Enable(GL::DEPTH_TEST);
    GL::Enable(GL::AUTO_NORMAL);

    $theNurb = GLU::NewNurbsRenderer();

    GLU::NurbsProperty($theNurb,GLU::SAMPLING_TOLERANCE, 25.0);
    GLU::NurbsProperty($theNurb,GLU::DISPLAY_MODE, GLU::FILL);
end

display = proc {
    GL::Clear(GL::COLOR_BUFFER_BIT | GL::DEPTH_BUFFER_BIT);

    GL::PushMatrix();
    GL::Translate(4.0, 4.5, 2.5);
    GL::Rotate(220.0, 1.0, 0.0, 0.0);
    GL::Rotate(115.0, 0.0, 1.0, 0.0);
    GL::Translate(-4.0, -4.5, -2.5);

    GLU::BeginSurface($theNurb);
    GLU::NurbsSurface($theNurb, S_NUMKNOTS, $sknots, T_NUMKNOTS, $tknots,
	    4 * T_NUMPOINTS, 4,
	    $ctlpoints.flatten,
	    S_ORDER, T_ORDER,
	    GL::MAP2_VERTEX_4);
    GLU::EndSurface($theNurb);

    GL::PopMatrix();
    GL::Flush();
}

myReshape = proc {|w, h|
    GL::Viewport(0, 0, w, h);
    GL::MatrixMode(GL::PROJECTION);
    GL::LoadIdentity();
    GL::Frustum(-1.0, 1.0, -1.5, 0.5, 0.8, 10.0);

    GL::MatrixMode(GL::MODELVIEW);
    GL::LoadIdentity();
    GLU::LookAt(7.0,4.5,4.0, 4.5,4.5,2.0, 6.0,-3.0,2.0);
}

#/*  Main Loop
# *  Open window with initial window size, title bar, 
# *  RGBA display mode, and handle input events.
# */
    GLUT::Init();
    GLUT::InitDisplayMode (GLUT::SINGLE | GLUT::RGB | GLUT::DEPTH);
    GLUT::CreateWindow ();
    myinit();
    GLUT::ReshapeFunc (myReshape);
    GLUT::DisplayFunc(display);
    GLUT::MainLoop();
